﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace FSM_Puppies.Game
{
    public abstract class GameObject
    {
        public PlayingField Owner { get; set; }
        public MotiveBase Motives { get; set; }

        /// <summary>
        /// Initializes a new instance of the GameObject class.
        /// </summary>
        public GameObject(PlayingField owner)
        {
            Owner = owner;
            Motives = new MotiveBase();
        }

        /// <summary>
        /// Update the object
        /// </summary>
        /// <param name="seconds">seconds since last update.</param>
        /// <remarks>seconds is the easiest scale for the individual settings</remarks>
        public abstract void Update(float seconds);

        /// <summary>
        /// Location on the playing field to draw the actor
        /// </summary>
        public PointF Location { get; set; }

        /// <summary>
        /// What to draw on the playing field
        /// </summary>
        public abstract Image Image { get; }

        /// <summary>
        /// Push the game object toward a location.  Default behavior is to not move.
        /// </summary>
        /// <param name="destination">Location to push toward</param>
        /// <param name="seconds">Seconds that have passed in this movement</param>
        public virtual void PushToward(PointF destination, float seconds) { return; }

        /// <summary>
        /// Get the maximim speed of this game object. Default behavior is not to move.
        /// </summary>
        /// <returns></returns>
        public virtual float MaxSpeed() { return 0; }

    }
}
