﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace FSM_Puppies.Game
{
    public partial class PlayingField : UserControl
    {
        public List<GameObject> GameObjects = new List<GameObject>();
        public Random Rng = new Random();
        DateTime lastFrame = DateTime.Now;


        public PlayingField()
        {
            InitializeComponent();
        }

        public void AddGameObject(GameObject obj)
        {
            PointF randomLocation = new PointF((float)Rng.NextDouble() * Width, (float)Rng.NextDouble() * Height);
            obj.Location = randomLocation;
            GameObjects.Remove(obj);   // Always remove before add to prevent duplicates.
            GameObjects.Add(obj);
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            DateTime now = DateTime.Now;
            long elapsed = now.Ticks - lastFrame.Ticks;

            // Convert from ticks to seconds.
            float dt = elapsed / (float)10000000;

            // Prevent huge time differences when we pause for debugging.
            if (dt > 0.1f)
                dt = 0.1f;
            foreach (GameObject obj in GameObjects)
            {
                obj.Update(dt);

                // If they leave the playing field reset them to a random location.
                if(obj.Location.X < 0
                    || obj.Location.X > Width
                    || obj.Location.Y < 0
                    || obj.Location.Y > Height)
                {
                    PointF randomLocation = new PointF((float)Rng.NextDouble() * Width, (float)Rng.NextDouble() * Height);
                    obj.Location = randomLocation;
                }
            }

            lastFrame = now;
            Invalidate();
            timer.Start();
        }

        private void PlayingField_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            g.Clear(BackColor);

            foreach (GameObject obj in GameObjects)
            {
                g.DrawImage(obj.Image, obj.Location);
            }
        }
    }
}
