﻿using System;
using System.Collections.Generic;

// In this simple example the main loop will work with a simple state machine.
// It will print the name of the current state,
// then print the results of running the state,
// then advance to the next state and print the new state's name
// This will repeat until the machine is done.

namespace SimpleStateMachine
{
    class SimpleStateMachine
    {
        static void Main(string[] args)
        {
            Console.WriteLine("Simple state machine example, by Bryan Wagstaff (aka frob)");

            Console.WriteLine("Creating state machine");
            StateMachine machine = new StateMachine();

            Console.WriteLine("Running state machine...");

            while (machine.IsDone() == false)
            {
                Console.WriteLine("Currently in " + machine.GetStateName());
                Console.WriteLine(machine.UpdateState());
                machine.NextState();
                Console.WriteLine("Advaned to " + machine.GetStateName());
            }

            Console.WriteLine("Press enter to continue");
            Console.ReadKey(true);
        }
    }


    public class StateMachine
    {
        #region Members
        public enum State
        {
            Routing,
            Sentrying,
            Attacking,
            Ending
        }
        State mState = State.Routing;
        Random rng = new Random();        
        #endregion

        public string GetStateName()
        {
            return mState.ToString();
        }
        public string UpdateState()
        {
            return "Running state " + GetStateName() +". Your game logic goes here.";
        }
        public void NextState()
        {
            switch (mState)
            {
                case State.Routing:
                    mState = State.Sentrying;
                    break;
                case State.Sentrying:
                    mState = State.Attacking;
                    break;
                case State.Attacking:
                    // TODO: Make this based on game logic instead of random name generator
                    if (rng.NextDouble() < 0.75)
                    {
                        Console.WriteLine("Random generator says NPC has survived.");
                        mState = State.Routing;
                    }
                    else
                    {
                        Console.WriteLine("Random generator says NPC did not survive.");
                        mState = State.Ending;
                    }
                    break;
                case State.Ending:
                    // Nothing to do.
                    break;
            }            
        }
        public bool IsDone()
        {
            return mState == State.Ending;
        }

                
    }
}
